# Glossary

This is a collection of commonly used terms when talking about Flux
and Fluorine.

## State

This term describes all data that is managed by your state manager, in this
case Fluorine. It is unfortunately not a very precise term, but generally
it is used to describe either the data in a single store, or that
in all of the stores.

## Actions

An **action** represents a mutation of your state that is managed by the state
manager. No matter how you want to manipulate your state, the change is
always described by an action.

Having exclusively pure objects as actions is considered good practice. The
objects must contain a `type` attribute. This attribute should in most
cases just be a string, since it's human readable and serializable for JSON.

Actions are dispatched on the dispatcher and consumed by reducers.

## Action Creator

An **action creator** is a function that returns a new action based on its
arguments. It can be wrapped so that its output is fed directly into a
dispatcher.

## Thunks

A **thunk** is a function returned from an action creator. When dispatched on
a dispatcher it is given arguments to dispatch or reduce from the dispatcher.
Thus it is a pattern for primitive side effects, or for generating agendas
from reducing the dispatcher's state.

## Agendas

An **agenda** is an Observable that emits actions. It is Fluorine's preferred
side effect model.

## Reducer

A **reducer** is a pure function that describes your store's state and its
possible mutations. A reducer takes a store's previous state and an action, and
returns the store's next state.

At all times, the reducer should be free of side effects. It should just return
a new immutable state.

## Store

In Flux a **store** is an object that keeps track of state. In Fluorine the
store is fortunately something very specific: It's an Observable. It emits
your store's up-to-date state, when it changes.

Stores are generated by reducing the dispatcher using a reducer.

The Observable that is generated always emits the last known state immediately
on subscription. Afterwards it emits when the state changes.

## Dispatcher

This is your application's central event hub, your central source of truth.
You can dispatch actions, thunks and agendas on it.
All stores are reduced from a dispatcher.

## Middleware

A **middleware** is a function that transforms agendas before they reach the
dispatcher, and thus the stores. They can transform the agenda in any given way
as long as the result is still an observable.

